<?php
namespace Zeedhi\Translate\Controller;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\DevOps\Model\Translator\Entities\ZhLanguage;
use Zeedhi\DevOps\Model\Translator\Entities\ZhRepository;
use Zeedhi\DevOps\Model\Translator\Entities\ZhRepositoryLanguage;

class Repository extends Crud{

    protected $dataSourceName = 'repository';
    protected $entityManager;

    public function __construct(Manager $dataSourceManager, $entityManager) {
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }

    protected function saveRepositoryLanguages($repository, $selectedLanguages = array()){
        $filter = array("repository" => $repository->getId());
        $repLangs = $this->entityManager->getRepository(ZhRepositoryLanguage::class)->findBy($filter);
        foreach($repLangs as $repLang){
            $id = $repLang->getLanguage()->getId();
            $selected = array_search($id, $selectedLanguages);
            if($selected === false){
                $this->entityManager->remove($repLang);
            } else {
                unset($selectedLanguages[$selected]);
            }
        }
        foreach($selectedLanguages as $languageId){
            $lang = $this->entityManager->find(ZhLanguage::class, $languageId);
            $repLang = new ZhRepositoryLanguage;
            $repLang->setLanguage($lang);
            $repLang->setRepository($repository);
            $this->entityManager->persist($repLang);
        }
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->entityManager->beginTransaction();
        try {
            $row = $request->getDataSet()->getRows()[0];
            $repositoryUrl = $row['URL'];
            $filter = array("url" => $repositoryUrl);
            $repository = $this->entityManager->getRepository(ZhRepository::class)->findOneBy($filter);
            if(!isset($repository)){
                $repository = new ZhRepository();
                $repository->setUrl($repositoryUrl);
            }

            $repository->setDefaultLanguage($row['DEFAULT_LANGUAGE']);
            $repository->setFrontPath($row['FRONT_PATH']);
            $repository->setIsPublic($row['IS_PUBLIC']);
            $repository->setNewStructure($row['NEW_STRUCTURE']);
            $repository->setWordsPath($row['WORDS_PATH']);
            $repository->setMessagesFile($row['MESSAGES_FILE']);
            $repository->setMessagesStructure($row['MESSAGES_STRUCTURE']);
            $this->entityManager->persist($repository);
            $this->saveRepositoryLanguages($repository, $row['LANGUAGES']);
            $this->entityManager->flush();
            $this->entityManager->commit();

            $updatedRepositories = array();
            $updatedRepositories[$repositoryUrl] = $repository->getId();
            $response->addDataSet(new DataSet($this->dataSourceName, $updatedRepositories));
        } catch(\Exception $e){
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $realDataSourceName = $this->dataSourceName;
        $this->dataSourceName = 'repositoryWithLanguages';
        parent::find($request, $response);
        $responseRows = $response->getDataSets()[0]->getRows();
        foreach($responseRows as &$row){
            if(!is_null($row['LANGUAGES'])){
                $row['LANGUAGES'] = array_map('intval', explode(',', $row['LANGUAGES']));
            }
        }
        $response->setDataSets(array(new DataSet($realDataSourceName, $responseRows)));
    }

    public function findUnsavedRepositories(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'unsavedRepositories';
        parent::find($request, $response);
    }

}