function LibLoginController(LibLoginService, LibUtilities, localStorageEngine, templateManager, eventAggregator, LibMessages,
                            ScreenService, metaDataFactory, WidgetsMetaData, ZhuAclController, ZhLogger, ZHPromise) {
    var self = this;
    var loginUserId, loginUserName;

    var factoryExpiredSessionWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getExpiredSession(), templateManager.container);
    };

    var getUrlParameter = function(name) {
        var searchString = window.location.search.substring(1);
        var parameters   = searchString.split('&');
        var result = '';
        parameters.forEach(function(parameter) {
            var splitedParameter = parameter.split('=');
            if(splitedParameter[0] === name) {
                result = splitedParameter[1];
            }
        });
        return result;
    };

    this.authenticate = function(forceAuthenticate) {
        templateManager.project.elementListDefaultRule = templateManager.project.elementListDefaultRule || 'I';
        var language = localStorage.getItem('LOGIN_LANGUAGE') || window.lang;
        window.lang = language;
        ScreenService.setLanguage(language);
        localStorageEngine.setLocalVar('language', language);

        var defer = ZHPromise.defer();
        forceAuthenticate = forceAuthenticate === true;

        if(window.location.hash.replace('#/', '') != LibUtilities.authenticationContainer || forceAuthenticate) {
            var setAclEventId;
            var user = localStorage.getItem('LOGIN_USER') || getUrlParameter('loginUser');
            var redirectUrl = localStorage.getItem('LOGIN_REDIRECT_URL');

            if(redirectUrl && !redirectUrl.includes(window.location.hostname)) {
                if (window.ZhNativeInterface) {
                    window.ZhNativeInterface.redirect(redirectUrl);
                } else {
                    window.location = redirectUrl;
                }
                defer.resolve({});
            } else if(!user) {
                LibUtilities.redirectToLogin();
                defer.resolve({});
            } else {
                var setAclFunction = function () {
                    setAcl(defer.promise, setAclEventId);
                };

                setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);

                LibLoginService.startSession(user).then(function (response) {
                    startSessionResponse(response, defer);
                }).catch(function (error) {
                    //Expired Token
                    if(error.data && error.data.errorCode == 101) {
                        self.logout();
                    }
                    //Invalid Token
                    else if(error.data && error.data.errorCode == 100) {
                        self.logout();
                    } else {
                        defer.reject(error);
                        LibUtilities.showErrorMessage(error);
                    }
                });
            }
        }

        return defer.promise;
    };

    var startSessionResponse = function(response, defer) {
        if (response.isError) {
            defer.reject(response.message);
        } else {
            if (!response.USER_ID) {
                self.logout();
            } else if (response.SESSION_EXPIRED) {
                logoutExpiredSession(true);
            } else {
                localStorage.setItem('IS_LOGGED', true);
                localStorage.setItem('LOGIN_USER', response.LOGIN_USER);
                localStorage.setItem('LOGIN_TOKEN', response.LOGIN_TOKEN);

                loginUserId    = response.USER_ID;
                loginUserName  = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    ScreenService.showMessage(ScreenService.i18n(LibMessages.ALE_MIGRATION_IS_PENDENT));
                }

                if (response.MODULES_VERSION_ERROR) {
                    ZhLogger.warning(response.MODULES_VERSION_ERROR);
                }

                defer.resolve(response);
            }
        }
    };

    this.logout = function() {
        LibLoginService.logout().then(function () {
            localStorage.removeItem('IS_LOGGED');
            localStorage.removeItem('LOGIN_USER');
            localStorage.removeItem('LOGIN_TOKEN');
            localStorage.removeItem('LOGIN_REDIRECT_URL');
            LibUtilities.redirectToLogin();
        }).catch(LibUtilities.showErrorMessage);
    };

    var setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if(!window.productApplyAcl) {
                ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                    {
                        canConfigureAcl: response.ACL_ADMINISTRATOR,
                        rules          : response.ACL_ADMINISTRATOR ? [] : false,
                        defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                    }
                );
                templateManager.project.aclDefaultRule = 'A';
            }
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
        }).catch(LibUtilities.showErrorMessage);
    };

    eventAggregator.subscribe("LibExpiredSession", function () {
        if(!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            var expiredSessionWidget = factoryExpiredSessionWidget();
            expiredSessionWidget.attempts = 0;
            expiredSessionWidget.dataSource.data = [];
            expiredSessionWidget.newRow();
            expiredSessionWidget.currentRow.OPERATOR = loginUserId + ' - ' + loginUserName;
            ScreenService.openPopup(expiredSessionWidget);
            localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', true);

        }
    });

    eventAggregator.subscribe("LibInvalidToken", function (errorCode) {
        //Expired Token
        if(errorCode == 101) {
            var actions = [ {
                label: ScreenService.i18n("Ok"),
                code: self.logout
            } ];
            ScreenService.showCustomDialog(LibMessages.ALE_EXPIRED_SESSION, "information", null, actions);
        }
        //Invalid Token
        else if(errorCode == 100) {
            self.logout()
        }
    });

    var logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        var loadingMessage = afterLogin ? LibMessages.ERR_EXPIRED_SESSION_LOGIN : LibMessages.ERR_ATTEMPT_END;
        ScreenService.changeLoadingMessage(loadingMessage);
        ScreenService.showLoader();
        Util.doAsync(self.logout, 5000);
    };

    this.expiredSessionActionEventConfirm = function(args) {
        var expiredSessionWidget = args.owner.widget ? args.owner.widget : args.owner.field.widget;
        if(expiredSessionWidget.isValid()) {
            var password = expiredSessionWidget.currentRow.PASSWORD;
            LibLoginService.validateUser(loginUserId, password)
                .then(function(response) {
                    response = response.dataset.userValidation;
                    if(response.TOKEN_ERROR) {
                        loginTokenErrorValidation(response.TOKEN_ERROR);
                    } else {
                        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                        ScreenService.closePopup();
                    }
                }).catch(function() {
                    if(expiredSessionWidget.attempts >= 2) {
                        ScreenService.showMessage(LibMessages.ERR_ATTEMPT_PARTIAL.replace('%1', ++expiredSessionWidget.attempts), 'ALERT');
                        logoutExpiredSession();
                    } else {
                        ScreenService.showMessage(LibMessages.ERR_ATTEMPT_PARTIAL.replace('%1', ++expiredSessionWidget.attempts), 'ALERT');
                    }
                });
        }
    };

    this.expiredSessionActionEventCancel = function() {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        self.logout();
    };

    this.expiredSessionFieldOnKeyupPassword = function(args) {
        var key = window.event.keyCode;

        //13 = ENTER
        if(key == 13) {
            this.expiredSessionActionEventConfirm(args);
        }
    };

    this.downloadModulesVersionFile = function() {
        return LibLoginService.generateModulesVersionFile().then(function(response) {
            if(response && response.dataset) {
                var fileName = response.dataset.generatedFile[0];
                LibLoginService.downloadModulesVersionFile(fileName);
                return true;
            } else {
                LibUtilities.showErrorMessage(response);
                return false;
            }
        }).catch(LibUtilities.showErrorMessage);
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginController', LibLoginController);
});