function LibUtilities(ScreenService, eventAggregator, templateManager, requestDataSourceEngine, restEngine) {
    var self = this;
    this.authenticationContainer    = 'login#authentication';
    this.expiredSessionMethod = null;

    this.showErrorMessage = function(error) {
        var message = error;
        if(typeof error === 'object') {
            if(error.message) {
                message = error.message;
            } else if(error.data && error.data.error){
                message = error.data.error;
            }
        }

        if(!message) {
            message = 'Ocorreu um erro inesperado. Entre em contato com o administrador do sistema.';
        } else if(message.indexOf('Exception: Exception: ') != -1) {
            message = message.replace('Exception: Exception: ', '');
        }

        ScreenService.showMessage(message, 'error');
    };

    this.redirectToLogin = function() {
        if(window.openLoginContainerWithoutRedirect) {
            ScreenService.openWindow(self.authenticationContainer);
        } else {
            self.openWindow(self.authenticationContainer);
        }
    };

    eventAggregator.subscribe("EventOnRequestStart", function (eventData) {
        var validateExpiredSessionRoute = 'lib_validateExpiredSession';
        var logoutRoute = 'lib_logout';
        var loginRoute  = 'login';
        var validateUserRoute = 'validateUser';

        var requestConfig = eventData.requestConfig;
        if (requestConfig && requestConfig.url && shouldAddRequestInfo(requestConfig.url) && templateManager.container) {
            requestConfig.data.origin = requestConfig.data.origin || {};
            var widget;
            var container = templateManager.loadingContainerJson;
            if(container) {
                widget = getFirstWidgetWithDataSourceRest(container.widgets);
                if(!widget) {
                    widget = container.widgets[0];
                }
            } else {
                container = templateManager.container;
                widget = container.getActiveWidget();
            }

            requestConfig.data.origin.containerLabel = container.label || container.name;
            requestConfig.data.origin.widgetLabel = widget.label || widget.name;
        }

        var route = requestConfig.url.split('/').pop();
        if(route != validateExpiredSessionRoute) {
            factoryExpiredSessionMethods();
        }
        if(route != logoutRoute && route != loginRoute) {
            var token = localStorage.getItem('LOGIN_TOKEN');
            var keepConnected = localStorage.getItem('LOGIN_KEEP_CONNECTED');
            if(window.externalTokens) {
                window.externalTokens.some(function(externalToken) {
                    if(eventData.requestConfig.url.includes(externalToken.url)) {
                        token = externalToken.token;
                        keepConnected = externalToken.keepConnected || keepConnected;
                        return true;
                    }
                    return false;
                });
            }
            if (token) {
                requestConfig.headers['OAuth-Token'] = token;
            }
            if(route == validateUserRoute) {
                keepConnected = 'Yes';
            }
            requestConfig.headers['OAuth-KeepConnected'] = keepConnected;
        }
    });

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
            !url.match(/.html/i) &&
            !url.match(/.svg/i);
    };

    var getFirstWidgetWithDataSourceRest = function(widgets) {
        for(var i = 0; i < widgets.length; i++) {
            if(widgets[i].dataSource && widgets[i].dataSource.rest == true) {
                return widgets[i];
            } else {
                return getFirstWidgetWithDataSourceRest(widgets[i]);
            }
        }
    };

    var factoryExpiredSessionMethods = function() {
        if(templateManager.project && templateManager.project.onRequestIdleTimeout) {
            if(self.expiredSessionMethod) {
                clearTimeout(self.expiredSessionMethod);
            }
            self.expiredSessionMethod = Util.doAsync(validateExpiredSession, templateManager.project.onRequestIdleTimeout);
        }
    };

    var validateExpiredSession = function() {
        var timeout = templateManager.project.onRequestIdleTimeout;
        var filter = [ {
            name: 'TIMEOUT',
            value: timeout,
            operator: '='
        } ];

        requestDataSourceEngine.getDataSource('lib#/lib_validateExpiredSession', filter).then(function(response){
            if(response && response.dataset && response.dataset.expiredSessionInfo) {
                var expiredSessionInfo = response.dataset.expiredSessionInfo;
                if(expiredSessionInfo.IS_EXPIRED) {
                    eventAggregator.publish('LibExpiredSession');
                } else {
                    if(self.expiredSessionMethod) {
                        clearTimeout(self.expiredSessionMethod);
                    }
                    Util.doAsync(validateExpiredSession, expiredSessionInfo.TIMEOUT);
                }
            } else {
                self.showErrorMessage(response);
            }
        }).catch(self.showErrorMessage);
    };

    this.changeLoginResponseToOldParams = function(loginResponse) {
        if(loginResponse) {
            return {
                cdoperador:          loginResponse.USER_ID,
                nmoperador:          loginResponse.USER_NAME,
                dsemailoper:         loginResponse.EMAIL,
                nrorg:               loginResponse.ORGANIZATION_ID,
                nmorg:               loginResponse.TRADING_NAME,
                nmrazsocialorg:      loginResponse.COMPANY_NAME,
                perfilDeAcesso:      loginResponse.ACCESS_PROFILE,

                cdempresa:           loginResponse.CDEMPRESA     || null,
                cdfilialselecionada: loginResponse.CDFILIAL      || null,
                cdpictprat:          loginResponse.CDPICTPRAT    || null,
                cdpictprod:          loginResponse.CDPICTPROD    || null,
                ididioma:            loginResponse.IDIDIOMA      || null,
                nmacesso:            loginResponse.NMACESSO      || null,
                nmcomploper:         loginResponse.NMCOMPLOPER   || null,
                nmfilialselecionada: loginResponse.NMFILIAL      || null,
                nrestruttrab:        loginResponse.NRESTRUTTRAB  || null,
                nrorgtrab:           loginResponse.NRORGTRAB     || null,
                nrparcnegocio:       loginResponse.NRPARCNEGOCIO || null,

                logged:              loginResponse.LOGGED,
                senhaExpirada:       loginResponse.EXPIRED_PASSWORD,
                version:             loginResponse.VERSION
            };
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    this.openMicroServiceUrl = function(args) {
        var containerName = args.owner.msContainerName;
        var currentModule, module;

        if(containerName.indexOf('#') == -1) {
            currentModule = module = '';
        } else {
            var subModule = containerName.split('#')[0];
            module        = subModule.split('_')[0];
            currentModule = getCurrentModule();
        }

        if (module && currentModule && (module != currentModule)) {
            var filter = [ {
                name: 'USER',
                value: localStorage.getItem('LOGIN_USER'),
                operator: '='
            }, {
                name: 'IS_LOGGED',
                value: localStorage.getItem('IS_LOGGED') || false,
                operator: '='
            }, {
                name: 'KEEP_CONNECTED',
                value: localStorage.getItem('LOGIN_KEEP_CONNECTED') || 'No',
                operator: '='
            } ];

            requestDataSourceEngine.getDataSource(module + '#/lib_startSession', filter).then(function() {
                ScreenService.openLink(args.owner.url);
            }).catch(self.showErrorMessage);
        }
    };

    this.openLinkByMenu = function(menu) {
        if(typeof menu === 'string') {
            menu = templateManager.project.getMenu(menu);
        }

        var args = { owner: {
            msContainerName: menu.msContainerName,
            url: menu.url
        } };
        self.openMicroServiceUrl(args);
    };

    this.openWindow = function(windowName, newWindow) {
        requestModulesMetadata().then(function (modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            if (ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            } else {
                ScreenService.openWindow(windowName);
            }
        });
    };

    var requestModulesMetadata = function () {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };

    eventAggregator.subscribe("AboutOnClick", function () {
        if(!templateManager.versionData) {
            templateManager.versionData = true;
            var filter = [ {
                name: 'CURRENT_MODULE',
                operator: '=',
                value: getCurrentModule()
            } ];
            requestDataSourceEngine.getDataSource('lib#/lib_getVersionInfo', filter).then(function (response) {
                templateManager.versionData = response.dataset.versionInfo;
                templateManager.versionData.dependencies.current = templateManager.versionData.dependencies.frontend;
                templateManager.versionData.dependencies.changeCurrentVersion = function(newVersion) {
                    templateManager.versionData.dependencies.current = templateManager.versionData.dependencies[newVersion];
                }
            }).catch(self.showErrorMessage);
        }
    });

    eventAggregator.subscribe("EventOnRequestError", function (eventData) {
        if(eventData.e && eventData.e.e && eventData.e.e.data && eventData.e.e.data.data) {
            var error = eventData.e.e.data.data;
            //Expired or Invalid Token
            if(error.errorCode == 101 || error.errorCode == 100) {
                eventAggregator.publish('LibInvalidToken', error.errorCode);
            }
        }
    });
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibUtilities', LibUtilities);
});
